@@ -37,8 +37,7 @@
          stream_chunk_end/1]).
 -export([stream_process_deliver/2, stream_process_deliver_chunk/2,
          stream_process_deliver_final_chunk/2, stream_process_end/2]).
--export([websocket_send/2, websocket_receive/1,
-         websocket_unframe_data/1, websocket_setopts/2]).
+-export([websocket_send/2]).
 -export([new_cookie_session/1, new_cookie_session/2, new_cookie_session/3,
          cookieval_to_opaque/1, request_url/1,
          print_cookie_sessions/0,
@@ -355,7 +354,7 @@ parse_arg_value([C|Line], Key, Value, Quote, _) ->
 %%
 
 make_parse_line_reply(Key, Value, Rest) ->
-    {{list_to_atom(yaws:funreverse(Key, {yaws, to_lowerchar})),
+    {{list_to_atom(yaws:funreverse(Key, fun yaws:to_lowerchar/1)),
       lists:reverse(Value)}, Rest}.
 
 
@@ -478,7 +477,7 @@ parse_multi(Data, #mp_parse_state{state=start_header}=ParseState, Acc, [], []) -
             parse_multi(Data, NParseState, Acc, [], [])
     end;
 parse_multi(Data, #mp_parse_state{state=header}=ParseState, Acc, Name, Hdrs) ->
-    case erlang:decode_packet(httph_bin, Data, []) of
+    case erlang:decode_packet(httph_bin, Data, [{packet_size, 16#4000}]) of
         {ok, http_eoh, Rest} ->
             Head = case Name of
                        [] ->
@@ -818,12 +817,14 @@ path_norm(Path) ->
 path_norm_reverse("/" ++ T) -> start_dir(0, "/", T);
 path_norm_reverse(       T) -> start_dir(0,  "", T).
 
-start_dir(N, Path, [$\\|T]    ) -> start_dir(N, Path, [$/|T]);
-start_dir(N, Path, ".."       ) -> rest_dir(N, Path, "");
-start_dir(N, Path, "/"   ++ T ) -> start_dir(N    , Path, T);
-start_dir(N, Path, "./"  ++ T ) -> start_dir(N    , Path, T);
-start_dir(N, Path, "../" ++ T ) -> start_dir(N + 1, Path, T);
-start_dir(N, Path,          T ) -> rest_dir (N    , Path, T).
+start_dir(N, Path, [$\\|T]     ) -> start_dir(N, Path, [$/|T]);
+start_dir(N, Path, ".."        ) -> rest_dir(N, Path, "");
+start_dir(N, Path, "/"    ++ T ) -> start_dir(N    , Path, T);
+start_dir(N, Path, "./"   ++ T ) -> start_dir(N    , Path, T);
+start_dir(N, Path, ".\\"  ++ T ) -> start_dir(N    , Path, T);
+start_dir(N, Path, "../"  ++ T ) -> start_dir(N + 1, Path, T);
+start_dir(N, Path, "..\\" ++ T ) -> start_dir(N + 1, Path, T);
+start_dir(N, Path,           T ) -> rest_dir (N    , Path, T).
 
 rest_dir (_N, Path, []         ) -> case Path of
                                         [] -> "/";
@@ -921,6 +922,13 @@ mime_type(FileName) ->
 stream_chunk_deliver(YawsPid, Data) ->
     YawsPid  ! {streamcontent, Data}.
 
+
+%% Use timeout here to guard against bug in the SSL application
+%% that apparently does not close the socket in between
+%% ssl_esock and erlang (FIN_WAIT2 resp. CLOSE_WAIT).
+%% Thus, the stream process hangs forever...
+-define(STREAM_GARBAGE_TIMEOUT, 3600000). % 1 hour
+
 %% Synchronous (on ultimate gen_tcp:send) delivery
 %% Returns: ok | {error, Rsn}
 stream_chunk_deliver_blocking(YawsPid, Data) ->
@@ -938,6 +946,16 @@ stream_chunk_deliver_blocking(YawsPid, Data) ->
             end;
         {'DOWN', Ref, _, _, Info} ->
             {error, {ypid_crash, Info}}
+    after ?STREAM_GARBAGE_TIMEOUT ->
+            %% Killing (unless this function is caught) process tree but
+            %% NOTE that as this is probably due to the OTP SSL application
+            %% not managing to close the socket (FIN_WAIT2
+            %% resp. CLOSE_WAIT) the SSL process is not killed (it traps
+            %% exit signals) and thus we will leak one file descriptor.
+	    error_logger:error_msg(
+	      "~p:stream_chunk_deliver_blocking/2 STREAM_GARBAGE_TIMEOUT "
+	      "(default 1 hour). Killing ~p", [?MODULE, YawsPid]),
+	    erlang:error(stream_garbage_timeout, [YawsPid, Data])
     end.
 
 stream_chunk_end(YawsPid) ->
@@ -976,37 +994,9 @@ stream_process_end(Sock, YawsPid) ->
     YawsPid ! endofstreamcontent.
 
 
-websocket_send(Socket, IoList) ->
-    DataFrame = [0, IoList, 255],
-    case Socket of
-	{sslsocket,_,_} ->
-	    ssl:send(Socket, DataFrame);
-	_ ->
-	    gen_tcp:send(Socket, DataFrame)
-    end.
-
-websocket_receive(Socket) ->
-    R = case Socket of
-	    {sslsocket,_,_} ->
-		ssl:recv(Socket, 0);
-	    _ ->
-		gen_tcp:recv(Socket, 0)
-	end,
-    case R of
-	{ok, DataFrames} ->
-	    ReceivedMsgs = yaws_websockets:unframe_all(DataFrames, []),
-	    {ok, ReceivedMsgs};
-	_ -> R
-    end.
-
-websocket_unframe_data(DataFrameBin) ->
-    {ok, Msg, <<>>} = yaws_websockets:unframe_one(DataFrameBin),
-    Msg.
-
-websocket_setopts({sslsocket,_,_}=Socket, Opts) ->
-    ssl:setopts(Socket, Opts);
-websocket_setopts(Socket, Opts) ->
-    inet:setopts(Socket, Opts).
+%% Pid must the the process in control of the websocket connection.
+websocket_send(Pid, {Type, Data}) ->
+    yaws_websockets:send(Pid, {Type, Data}).
 
 
 %% Return new cookie string
@@ -1373,11 +1363,12 @@ is_abs_URI1(_) ->
 %% ------------------------------------------------------------
 %% simple erlang term representation of HTML:
 %% EHTML = [EHTML] | {Tag, Attrs, Body} | {Tag, Attrs} | {Tag} |
+%%         {Module, Fun, [Args]} | fun/0
 %%         binary() | character()
-%% Tag          = atom()
+%% Tag   = atom()
 %% Attrs = [{Key, Value}]  or {EventTag, {jscall, FunName, [Args]}}
-%% Key          = atom()
-%% Value = string()
+%% Key   = atom()
+%% Value = string() | {Module, Fun, [Args]} | fun/0
 %% Body  = EHTML
 
 ehtml_expand(Ch) when Ch >= 0, Ch =< 255 -> Ch; %yaws_api:htmlize_char(Ch);
@@ -1391,7 +1382,6 @@ ehtml_expand({ssi,File, Del, Bs}) ->
             X
     end;
 
-
 %%!todo (low priority) - investigate whether tail-recursion would be of any
 %% benefit here instead of the current ehtml_expand(Body) recursion.
 %%                - provide a tail_recursive version & add a file in the
@@ -1400,6 +1390,9 @@ ehtml_expand({ssi,File, Del, Bs}) ->
 ehtml_expand({Tag}) ->
     ["<", atom_to_list(Tag), " />"];
 ehtml_expand({pre_html, X}) -> X;
+ehtml_expand({Mod, Fun, Args})
+  when is_atom(Mod), is_atom(Fun), is_list(Args) ->
+    ehtml_expand(Mod:Fun(Args));
 ehtml_expand({Tag, Attrs}) ->
     NL = ehtml_nl(Tag),
     [NL, "<", atom_to_list(Tag), ehtml_attrs(Attrs), "></",
@@ -1409,8 +1402,9 @@ ehtml_expand({Tag, Attrs, Body}) when is_atom(Tag) ->
     NL = ehtml_nl(Tag),
     [NL, "<", Ts, ehtml_attrs(Attrs), ">", ehtml_expand(Body), "</", Ts, ">"];
 ehtml_expand([H|T]) -> [ehtml_expand(H)|ehtml_expand(T)];
-ehtml_expand([]) -> [].
-
+ehtml_expand([]) -> [];
+ehtml_expand(Fun) when is_function(Fun) ->
+    ehtml_expand(Fun()).
 
 
 ehtml_attrs([]) -> [];
@@ -1418,26 +1412,37 @@ ehtml_attrs([Attribute|Tail]) when is_atom(Attribute) ->
     [[$ |atom_to_list(Attribute)]|ehtml_attrs(Tail)];
 ehtml_attrs([Attribute|Tail]) when is_list(Attribute) ->
     [" ", Attribute|ehtml_attrs(Tail)];
+ehtml_attrs([{Name, {Mod, Fun, Args}} | Tail])
+  when is_atom(Mod), is_atom(Fun), is_list(Args) ->
+    ehtml_attrs([{Name,  Mod:Fun(Args)} | Tail]);
+ehtml_attrs([{Name, Value} | Tail]) when is_function(Value) ->
+    ehtml_attrs([{Name, Value()} | Tail]);
 ehtml_attrs([{Name, Value} | Tail]) ->
-    ValueString = if is_atom(Value) -> [$",atom_to_list(Value),$"];
-                     is_list(Value) -> [$",Value,$"];
-                     is_integer(Value) -> [$",integer_to_list(Value),$"];
-                     is_float(Value) -> [$",float_to_list(Value),$"]
-                  end,
+    ValueString = [$", if
+                           is_atom(Value) -> atom_to_list(Value);
+                           is_list(Value) -> Value;
+                           is_integer(Value) -> integer_to_list(Value);
+                           is_float(Value) -> float_to_list(Value)
+                       end, $"],
     [[$ |atom_to_list(Name)], [$=|ValueString]|ehtml_attrs(Tail)];
+ehtml_attrs([{check, Name, {Mod, Fun, Args}} | Tail])
+  when is_atom(Mod), is_atom(Fun), is_list(Args) ->
+    ehtml_attrs([{check, Name,  Mod:Fun(Args)} | Tail]);
+ehtml_attrs([{check, Name, Value} | Tail]) when is_function(Value) ->
+    ehtml_attrs([{check, Name, Value()} | Tail]);
 ehtml_attrs([{check, Name, Value} | Tail]) ->
-    ValueString = if is_atom(Value) -> [$",atom_to_list(Value),$"];
-                     is_list(Value) ->
-                          Q = case deepmember($", Value) of
-                                  true -> $';
-                                  false -> $"
-                              end,
-                          [Q,Value,Q];
-                     is_integer(Value) -> [$",integer_to_list(Value),$"];
-                     is_float(Value) -> [$",float_to_list(Value),$"]
-                   end,
-                   [[$ |atom_to_list(Name)],
-                    [$=|ValueString]|ehtml_attrs(Tail)].
+    Val = if
+              is_atom(Value) -> atom_to_list(Value);
+              is_list(Value) -> Value;
+              is_integer(Value) -> integer_to_list(Value);
+              is_float(Value) -> float_to_list(Value)
+          end,
+    Q = case deepmember($", Val) of
+            true -> $';
+            false -> $"
+        end,
+    ValueString = [Q,Value,Q],
+    [[$ |atom_to_list(Name)], [$=|ValueString]|ehtml_attrs(Tail)].
 
 
 
@@ -2065,6 +2070,7 @@ embedded_start_conf(DocRoot, SL, GL, Id)
     %% (see for example the start of the yaws_session_server)
     ok = application:set_env(yaws, embedded_conf, [{sclist,SCList},{gc,GC}]),
 
+    yaws:mkdir(yaws:id_dir(Id)),
     {ok, SCList, GC, ChildSpecs ++ SoapChild}.
 
 
